@@ -187,6 +187,7 @@ main (int argc, char *argv[])
 	    buf[chars - 1] = '\0';
 
 	  result = handle_address (buf, dwfl);
+	  fflush (stdout);
 	}
 
       free (buf);
@@ -618,7 +619,7 @@ handle_address (const char *string, Dwfl *dwfl)
 	case 1:
 	  addr = 0;
 	  j = i;
-	  /* Fallthrough */
+	  FALLTHROUGH;
 	case 2:
 	  if (string[j] != '\0')
 	    break;
