@@ -1,5 +1,5 @@
 /* Create, modify, and extract from archives.
-   Copyright (C) 2005-2012, 2016 Red Hat, Inc.
+   Copyright (C) 2005-2012, 2016, 2017 Red Hat, Inc.
    This file is part of elfutils.
    Written by Ulrich Drepper <drepper@redhat.com>, 2005.
 
@@ -28,7 +28,6 @@
 #include <libintl.h>
 #include <limits.h>
 #include <locale.h>
-#include <search.h>
 #include <stdbool.h>
 #include <stdlib.h>
 #include <stdio.h>
@@ -437,26 +436,11 @@ copy_content (Elf *elf, int newfd, off_t off, size_t n)
   return write_retry (newfd, rawfile + off, n) != (ssize_t) n;
 }
 
-
-static int
-string_compare (const void *a, const void *b)
-{
-  return strcmp((const char *)a, (const char *)b);
-}
-
-
-void
-free_node (void *node)
-{
-  (void) node;
-}
-
-
 static int
 do_oper_extract (int oper, const char *arfname, char **argv, int argc,
 		 long int instance)
 {
-  bool found[argc];
+  bool found[argc > 0 ? argc : 1];
   memset (found, '\0', sizeof (found));
 
   size_t name_max = 0;
@@ -473,15 +457,6 @@ do_oper_extract (int oper, const char *arfname, char **argv, int argc,
   Elf *elf;
   int fd = open_archive (arfname, O_RDONLY | O_BINARY, 0, &elf, NULL, false);
 
-  void *root = NULL;
-
-  for (int cnt = 0; cnt < argc; ++cnt)
-    {
-      if (tsearch (argv[cnt], &root, &string_compare) == NULL)
-	error (EXIT_FAILURE, errno,
-	       gettext ("cannot insert into hash table"));
-    }
-
   struct stat st;
   if (force_symtab)
     {
@@ -519,10 +494,15 @@ do_oper_extract (int oper, const char *arfname, char **argv, int argc,
       bool do_extract = argc <= 0;
       if (!do_extract)
 	{
-	  void *res = tfind (arhdr->ar_name, &root, &string_compare);
-	  if (res != NULL && (instance < 0 || instance-- == 0)
-	      && !found[(char **) res - argv])
-	    found[(char **) res - argv] = do_extract = true;
+	  for (int cnt = 0; cnt < argc; ++cnt)
+	    {
+	      if (!strcmp (arhdr->ar_name, argv[cnt]))
+		{
+		  if ((instance < 0 || instance-- == 0) && !found[cnt])
+		    found[cnt] = do_extract = true;
+		  break;
+		}
+	    }
 	}
 
       if (do_extract)
@@ -648,8 +628,8 @@ do_oper_extract (int oper, const char *arfname, char **argv, int argc,
 	    {
 	      error (0, errno, gettext ("failed to write %s"), arhdr->ar_name);
 	      status = 1;
-	      unlink (tempfname);
 	      close (xfd);
+	      unlink (tempfname);
 	      goto next;
 	    }
 
@@ -667,6 +647,7 @@ do_oper_extract (int oper, const char *arfname, char **argv, int argc,
 
 	      /* If we used a temporary file, move it do the right
 		 name now.  */
+	      close (xfd);
 	      if (use_mkstemp)
 		{
 		  int r;
@@ -678,7 +659,10 @@ do_oper_extract (int oper, const char *arfname, char **argv, int argc,
 			unlink (tempfname);
 		    }
 		  else
-		    r = rename (tempfname, arhdr->ar_name);
+		    {
+		      unlink (arhdr->ar_name);
+		      r = rename (tempfname, arhdr->ar_name);
+		    }
 
 		  if (unlikely (r) != 0)
 		    {
@@ -700,7 +684,10 @@ do_oper_extract (int oper, const char *arfname, char **argv, int argc,
 				unlink (tempfname);
 			    }
 			  else
-			    r = rename (tempfname, truncfname);
+			    {
+			      unlink (truncfname);
+			      r = rename (tempfname, truncfname);
+			    }
 			}
 
 		      if (r != 0)
@@ -713,8 +700,6 @@ cannot rename temporary file to %.*s"),
 			}
 		    }
 		}
-
-	      close (xfd);
 	    }
 	}
 
@@ -724,9 +709,6 @@ cannot rename temporary file to %.*s"),
 	error (1, 0, "%s: %s", arfname, elf_errmsg (-1));
     }
 
-  tdestroy(root, &free_node);
-  root = NULL;
-
   if (force_symtab)
     {
       arlib_finalize ();
@@ -752,9 +734,9 @@ cannot rename temporary file to %.*s"),
 		{
 		  // XXX Use /prof/self/fd/%d ???
 		nonew_unlink:
-		  unlink (tmpfname);
 		  if (newfd != -1)
 		    close (newfd);
+		  unlink (tmpfname);
 		  goto nonew;
 		}
 
@@ -791,8 +773,8 @@ cannot rename temporary file to %.*s"),
 		  ({asm ("" :: "r" (fchown (newfd, st.st_uid,
 						st.st_gid))); }),
 #endif
-		      close (newfd) != 0) ||
-		  (newfd = -1, rename (tmpfname, arfname) != 0))
+		      close (newfd) != 0) || (newfd = -1, close (fd) != 0) ||
+		  (fd = -1, unlink (arfname), rename (tmpfname, arfname) != 0))
 		goto nonew_unlink;
 	    }
 	}
@@ -800,7 +782,8 @@ cannot rename temporary file to %.*s"),
 
   elf_end (elf);
 
-  close (fd);
+  if (fd != -1)
+    close (fd);
 
   not_found (argc, argv, found);
 
@@ -905,15 +888,6 @@ do_oper_delete (const char *arfname, char **argv, int argc,
   struct stat st;
   int fd = open_archive (arfname, O_RDONLY | O_BINARY, 0, &elf, &st, false);
 
-  void *root = NULL;
-
-  for (int cnt = 0; cnt < argc; ++cnt)
-    {
-      if (tsearch (argv[cnt], &root, &string_compare) == NULL)
-	error (EXIT_FAILURE, errno,
-	       gettext ("cannot insert into hash table"));
-    }
-
   arlib_init ();
 
   off_t cur_off = SARMAG;
@@ -931,10 +905,15 @@ do_oper_delete (const char *arfname, char **argv, int argc,
       bool do_delete = argc <= 0;
       if (!do_delete)
 	{
-	  void *res = tfind (arhdr->ar_name, &root, &string_compare);
-	  if (res != NULL && (instance < 0 || instance-- == 0)
-	      && !found[(char **) res - argv])
-	    found[(char **) res - argv] = do_delete = true;
+	  for (int cnt = 0; cnt < argc; ++cnt)
+	    {
+	      if (!strcmp (arhdr->ar_name, argv[cnt]))
+	        {
+	          if ((instance < 0 || instance-- == 0) && !found[cnt])
+		    found[cnt] = do_delete = true;
+	          break;
+	        }
+	    }
 	}
 
       if (do_delete)
@@ -975,9 +954,6 @@ do_oper_delete (const char *arfname, char **argv, int argc,
 
   arlib_finalize ();
 
-  tdestroy (root, &free_node);
-  root = NULL;
-
   /* Create a new, temporary file in the same directory as the
      original file.  */
   char tmpfname[strlen (arfname) + 7];
@@ -1045,18 +1021,17 @@ do_oper_delete (const char *arfname, char **argv, int argc,
       /* Never complain about fchown failing.  */
       ({asm ("" :: "r" (fchown (newfd, st.st_uid, st.st_gid))); }),
 #endif
-	close (newfd) != 0) ||
-      (newfd = -1, rename (tmpfname, arfname) != 0))
+	close (newfd) != 0) || (newfd = -1, close (fd) != 0) ||
+      (fd = -1, unlink (arfname), rename (tmpfname, arfname) != 0))
     goto nonew_unlink;
 
  errout:
-#ifdef DEBUG
   elf_end (elf);
 
   arlib_fini ();
 
-  close (fd);
-#endif
+  if (fd != -1)
+    close (fd);
 
   not_found (argc, argv, found);
 
@@ -1064,31 +1039,28 @@ do_oper_delete (const char *arfname, char **argv, int argc,
 }
 
 
-static void
+/* Prints the given value in the given buffer without a trailing zero char.
+   Returns false if the given value doesn't fit in the given buffer.  */
+static bool
 no0print (bool ofmt, char *buf, int bufsize, long int val)
 {
   char tmpbuf[bufsize + 1];
-  snprintf (tmpbuf, sizeof (tmpbuf), ofmt ? "%-*lo" : "%-*ld", bufsize, val);
+  int ret = snprintf (tmpbuf, sizeof (tmpbuf), ofmt ? "%-*lo" : "%-*ld",
+		      bufsize, val);
+  if (ret >= (int) sizeof (tmpbuf))
+    return false;
   memcpy (buf, tmpbuf, bufsize);
+  return true;
 }
 
-
-static int
-basename_compare(const void *a, const void *b)
-{
-  return strcmp(basename((const char *)a), basename((const char *)b));
-}
-
-
 static int
 do_oper_insert (int oper, const char *arfname, char **argv, int argc,
 		const char *member)
 {
   int status = 0;
-  Elf *elf;
+  Elf *elf = NULL;
   struct stat st;
   int fd = open_archive (arfname, O_RDONLY | O_BINARY, 0, &elf, &st, oper != oper_move);
-  void *root = NULL;
 
   /* List of the files we keep.  */
   struct armem *all = NULL;
@@ -1110,20 +1082,6 @@ do_oper_insert (int oper, const char *arfname, char **argv, int argc,
       goto no_old;
     }
 
-  /* Store the names of all files from the command line in a hash
-     table so that we can match it.  Note that when no file name is
-     given we are basically doing nothing except recreating the
-     index.  */
-  if (oper != oper_qappend)
-    {
-      for (int cnt = 0; cnt < argc; ++cnt)
-	{
-	  if (tsearch (argv[cnt], &root, full_path ? &basename_compare : &string_compare) == NULL)
-	    error (EXIT_FAILURE, errno,
-		   gettext ("cannot insert into hash table"));
-	}
-    }
-
   /* While iterating over the current content of the archive we must
      determine a number of things: which archive members to keep,
      which are replaced, and where to insert the new members.  */
@@ -1161,23 +1119,32 @@ do_oper_insert (int oper, const char *arfname, char **argv, int argc,
 	      member = NULL;
 	    }
 
-	  void *res = tfind(arhdr->ar_name, &root, full_path ? &basename_compare : &string_compare);
-	  if (res != NULL && found[(char **) res - argv] == NULL)
+	  const char *arhdr_basename = basename (arhdr->ar_name);
+	  for (int cnt = 0; cnt < argc; ++cnt)
 	    {
-	      found[(char **) res - argv] = newp;
+	      if (full_path && strcmp (arhdr_basename, basename (argv[cnt])))
+		continue;
+	      else if (!full_path && strcmp (arhdr->ar_name, argv[cnt]))
+		continue;
 
-	      /* If we insert before or after a certain element move
-		 all files to a special list.  */
-	      if (unlikely (ipos != ipos_none || oper == oper_move))
-		{
-		  if (after_memberelem == newp)
-		    /* Since we remove this element even though we should
-		       insert everything after it, we in fact insert
-		       everything after the previous element.  */
-		    after_memberelem = all;
+	      if (found[cnt] == NULL)
+	        {
+	          found[cnt] = newp;
 
-		  goto next;
+	          /* If we insert before or after a certain element move
+		     all files to a special list.  */
+	          if (unlikely (ipos != ipos_none || oper == oper_move))
+		    {
+		      if (after_memberelem == newp)
+		        /* Since we remove this element even though we should
+		           insert everything after it, we in fact insert
+		           everything after the previous element.  */
+		        after_memberelem = all;
+
+		      goto next;
+		    }
 		}
+	      break;
 	    }
 	}
 
@@ -1195,12 +1162,6 @@ do_oper_insert (int oper, const char *arfname, char **argv, int argc,
 	error (EXIT_FAILURE, 0, "%s: %s", arfname, elf_errmsg (-1));
     }
 
-  if (oper != oper_qappend)
-    {
-      tdestroy(root, &free_node);
-      root = NULL;
-    }
-
  no_old:
   if (member != NULL)
     error (EXIT_FAILURE, 0, gettext ("position member %s not found"),
@@ -1300,13 +1261,11 @@ do_oper_insert (int oper, const char *arfname, char **argv, int argc,
 
   if (status != 0)
     {
-#ifdef DEBUG
       elf_end (elf);
 
       arlib_fini ();
 
       close (fd);
-#endif
 
       return status;
     }
@@ -1398,9 +1357,9 @@ do_oper_insert (int oper, const char *arfname, char **argv, int argc,
       if (fd != -1)
 	{
 	  // XXX Use /prof/self/fd/%d ???
-	  unlink (tmpfname);
 	  if (newfd != -1)
 	    close (newfd);
+	  unlink (tmpfname);
 	}
     nonew:
       error (0, errno, gettext ("cannot create new file"));
@@ -1460,14 +1419,36 @@ do_oper_insert (int oper, const char *arfname, char **argv, int argc,
 		  memcpy (arhdr.ar_name, tmpbuf, sizeof (arhdr.ar_name));
 		}
 
-	      no0print (false, arhdr.ar_date, sizeof (arhdr.ar_date),
-			all->sec);
-	      no0print (false, arhdr.ar_uid, sizeof (arhdr.ar_uid), all->uid);
-	      no0print (false, arhdr.ar_gid, sizeof (arhdr.ar_gid), all->gid);
-	      no0print (true, arhdr.ar_mode, sizeof (arhdr.ar_mode),
-			all->mode);
-	      no0print (false, arhdr.ar_size, sizeof (arhdr.ar_size),
-			all->size);
+	      if (! no0print (false, arhdr.ar_date, sizeof (arhdr.ar_date),
+			      all->sec))
+		{
+		  error (0, errno, gettext ("cannot represent ar_date"));
+		  goto nonew_unlink;
+		}
+	      if (! no0print (false, arhdr.ar_uid, sizeof (arhdr.ar_uid),
+			      all->uid))
+		{
+		  error (0, errno, gettext ("cannot represent ar_uid"));
+		  goto nonew_unlink;
+		}
+	      if (! no0print (false, arhdr.ar_gid, sizeof (arhdr.ar_gid),
+			      all->gid))
+		{
+		  error (0, errno, gettext ("cannot represent ar_gid"));
+		  goto nonew_unlink;
+		}
+	      if (! no0print (true, arhdr.ar_mode, sizeof (arhdr.ar_mode),
+			all->mode))
+		{
+		  error (0, errno, gettext ("cannot represent ar_mode"));
+		  goto nonew_unlink;
+		}
+	      if (! no0print (false, arhdr.ar_size, sizeof (arhdr.ar_size),
+			all->size))
+		{
+		  error (0, errno, gettext ("cannot represent ar_size"));
+		  goto nonew_unlink;
+		}
 	      memcpy (arhdr.ar_fmag, ARFMAG, sizeof (arhdr.ar_fmag));
 
 	      if (unlikely (write_retry (newfd, &arhdr, sizeof (arhdr))
@@ -1512,18 +1493,20 @@ do_oper_insert (int oper, const char *arfname, char **argv, int argc,
 	  /* Never complain about fchown failing.  */
 	  ({asm ("" :: "r" (fchown (newfd, st.st_uid, st.st_gid))); }),
 #endif
-	    close (newfd) != 0) ||
-	  (newfd = -1, rename (tmpfname, arfname) != 0)))
+	    close (newfd) != 0) || (newfd = -1, close(fd) != 0) ||
+	  (fd = -1, unlink (arfname), rename (tmpfname, arfname) != 0)))
       goto nonew_unlink;
 
  errout:
-#ifdef DEBUG
+  for (int cnt = 0; cnt < argc; ++cnt)
+    elf_end (found[cnt]->elf);
+
   elf_end (elf);
 
   arlib_fini ();
 
-  close (fd);
-#endif
+  if (fd != -1)
+    close (fd);
 
   return status;
 }
